package main

import (
	"encoding/json"
	"fmt"
	"strconv"

	"github.com/beego/beego/v2/core/logs"
	"github.com/beego/beego/v2/server/web"
)

// 定义 Controller
type CityController struct {
	web.Controller
}

type CityForm struct {
	X int `form:"x"`
	Y int // URL 必须是大写的 Y 才能匹配到
}

// Get 获取URL参数和Form(application/x-www-form-urlencoded)中数据示例
func (c *CityController) Get() {
	// http://127.0.0.1:8080/city/?x=1&y=2
	// 方法一
	c.Ctx.WriteString(c.Ctx.Input.Query("x"))
	c.Ctx.WriteString(c.Ctx.Input.Query("y"))
	c.Ctx.WriteString("\n")

	// 方法二
	var x string
	c.Ctx.Input.Bind(&x, "x")
	c.Ctx.WriteString(x)
	c.Ctx.WriteString("\n")

	// 方式三
	c.Ctx.WriteString(c.GetString("x"))
	xi, _ := c.GetInt("x")
	c.Ctx.WriteString(strconv.Itoa(xi))
	c.Ctx.WriteString("\n")

	// 方式四
	c.Ctx.Request.ParseForm()
	c.Ctx.WriteString(c.Ctx.Request.Form.Encode())
	c.Ctx.WriteString("\n")

	// 方式五
	var form CityForm
	c.ParseForm(&form)
	c.Ctx.WriteString(fmt.Sprintf("%#v", form))
	c.Ctx.WriteString("\n")

	// 获取 url router 参数
	c.Ctx.WriteString(fmt.Sprintf("%#v", c.Ctx.Input.Params()))
	c.Ctx.WriteString(c.Ctx.Input.Param("x"))
	c.Ctx.WriteString(c.Ctx.Input.Param("y"))
	c.Ctx.WriteString("\n")
}

// Post request body 参数获取示例
func (c *CityController) Post() {
	// 获取 form 数据
	//c.Ctx.Request.ParseForm()
	c.Ctx.WriteString(c.Ctx.Request.Method)
	c.Ctx.WriteString(c.Ctx.Request.Form.Encode())
	c.Ctx.WriteString("\n")

	// 方式二：仅获取 Post Form 数据
	c.Ctx.WriteString(c.Ctx.Request.PostForm.Encode())
	c.Ctx.WriteString("\n")

	//方式三，使用较多
	c.Ctx.WriteString(c.GetString("name"))
	c.Ctx.WriteString("\n")

	// 方式四，使用较多
	var form CityForm
	c.ParseForm(&form)
	c.Ctx.WriteString(fmt.Sprintf("%#v", form))
	c.Ctx.WriteString("\n")
}

type FileController struct {
	web.Controller
}

// Post 处理 form-data 文件上传
func (c *FileController) Post() {
	// 方式一
	//c.Ctx.Request.FormFile("abc")
	files := c.Ctx.Request.MultipartForm.File
	logs.Info("%#v", files)

	// 方式二
	//file := c.GetFile("abc")
	//defer file.Close()

	// 方式三
	c.SaveToFile("abc", "abc.json")

	//c.SaveToFileWithBuffer(fromFile string, toFile string, buf []byte) error {

	c.Ctx.WriteString(fmt.Sprintf("%#v", files))
}

type JsonController struct {
	web.Controller
}

// Post 处理 application/json 文件上传
func (c *JsonController) Post() {
	c.Ctx.Input.CopyBody(1 * 1024 * 1024) // 1Mb，必须加该行，或app.conf中配置，否则拿不到数据
	m := make(map[string]interface{})
	err := json.Unmarshal(c.Ctx.Input.RequestBody, &m)
	if err != nil {
		c.Ctx.WriteString(err.Error())
	} else {
		c.Ctx.WriteString(fmt.Sprintf("%#v\n", m))
	}
}

func main() {
	web.Router("/city/", &CityController{})
	web.Router("/file/", &FileController{})
	web.Router("/json/", &JsonController{})

	web.Run()
}
